/************************************************************************
 *
 *  PageStyleConverter.java
 *
 *  Copyright: 2002-2014 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.4 (2014-09-16)
 *
 */

package writer2latex.xhtml.style;

import java.util.Enumeration;

import writer2latex.office.MasterPage;
import writer2latex.office.OfficeReader;
import writer2latex.office.OfficeStyleFamily;
import writer2latex.office.PageLayout;
import writer2latex.office.StyleWithProperties;
import writer2latex.office.XMLString;
import writer2latex.util.CSVList;
import writer2latex.util.Calc;
import writer2latex.xhtml.Converter;
import writer2latex.xhtml.StyleInfo;
import writer2latex.xhtml.XhtmlConfig;
import writer2latex.xhtml.style.properties.Properties;

/**
 * This class converts OpenDocument page styles to CSS2 styles.
 * A page style in a presentation is represented through the master page,
 * which links to a page layout defining the geometry and optionally a drawing
 * page defining the drawing background.
 * 
 * In a presentation document we export the full page style.
 * In a text document we export the writing direction, background color and footnote rule for the first master page only
 */
public class PageStyleParser extends StyleParser {
	
	private boolean hasFootnoteRules = false;

	/** Create a new <code>PageStyleConverter</code>
     *  @param ofr an <code>OfficeReader</code> to read style information from
     *  @param config the configuration to use
     *  @param converter the main <code>Converter</code> class
     *  @param nType the type of xhtml to use
     */
    public PageStyleParser(OfficeReader ofr, XhtmlConfig config, Converter converter, int nType) {
        super(ofr,config,converter,nType);
        this.bConvertStyles = config.xhtmlFormatting()==XhtmlConfig.CONVERT_ALL || config.xhtmlFormatting()==XhtmlConfig.IGNORE_HARD;
    }
    
    /** Get the text width of the first master page (page width minus left and right margin)
     * 
     *  @return the text width
     */
    public String getTextWidth() {
        MasterPage masterPage = ofr.getFirstMasterPage();
        if (masterPage!=null) {
            PageLayout pageLayout = ofr.getPageLayout(masterPage.getPageLayoutName());
            if (pageLayout!=null) {
                String sWidth = pageLayout.getProperty(XMLString.FO_PAGE_WIDTH);
                if (sWidth!=null) {
                	String sMarginLeft = pageLayout.getProperty(XMLString.FO_MARGIN_LEFT);
                	if (sMarginLeft!=null) {
                		sWidth = Calc.sub(sWidth, sMarginLeft);
                	}
                	String sMarginRight = pageLayout.getProperty(XMLString.FO_MARGIN_RIGHT);
                	if (sMarginRight!=null) {
                		sWidth = Calc.sub(sWidth, sMarginRight);
                	}
                	return sWidth;
                }
            }
        }
        return "17cm"; // Default in Writer, hence usable as a fallback value
    }
    
    /** Apply footnote rule formatting (based on first master page)
     * 
     * @param info then StyleInfo to which style information should be attached
     */
    public void applyFootnoteRuleStyle(StyleInfo info) {
    	hasFootnoteRules = true;
    	info.sClass="footnoterule";
    }
    
    /** Apply default writing direction (based on first master page)
     * 
     * @param info then StyleInfo to which style information should be attached
     */
    public void applyDefaultWritingDirection(StyleInfo info) {
        MasterPage masterPage = ofr.getFirstMasterPage();
        if (masterPage!=null) {
            PageLayout pageLayout = ofr.getPageLayout(masterPage.getPageLayoutName());
            if (pageLayout!=null) {
                applyDirection(pageLayout,info);
            }
        }    	
    }

    /** Apply a master page style - currently only for presentations
     * 
     * @param sStyleName The name of the master page
     * @param info the StyleInfo to which style information should be attached
     */
    public void applyStyle(String sStyleName, StyleInfo info) {
        MasterPage masterPage = ofr.getMasterPage(sStyleName);
        if (masterPage!=null) {
            String sDisplayName = masterPage.getDisplayName();
            if (ofr.isPresentation()) {
                // Always generates class name
            	styleNames.addName(sDisplayName);
              info.sClass="masterpage"+styleNames.getName(sDisplayName);
            }
        }
    }

    /** Convert style information for used styles
     *  @param sIndent a String of spaces to add before each line
     */
    public String composeStyleDeclarations() {
        StringBuilder buf = new StringBuilder();

        // This will be master pages for presentations only
        Enumeration<String> names = styleNames.keys();
        while (names.hasMoreElements()) {
            String sDisplayName = names.nextElement();
            buf.append(composeDeclaration(sDisplayName));
        }
        if (ofr.isText() && bConvertStyles) {
        	// Export page formatting for first master page in text documents
        	 buf.append(composeFirstPageDeclaration());
        }
        return buf.toString();
    }

	private String composeFirstPageDeclaration() {
		StringBuilder buf = new StringBuilder();
		MasterPage masterPage = ofr.getFirstMasterPage();
		if (masterPage == null) {	return ""; }
		PageLayout pageLayout = ofr.getPageLayout(masterPage.getPageLayoutName());
		if (pageLayout == null) {	return ""; }

		// Background color
		String pageDeclaration = composePageDeclaration();
		if (!pageDeclaration.isEmpty()) {
			buf.append(createDeclaration("body", pageDeclaration));
		}
		// Footnote rule
		String footnoteDeclaration = composeFootnoteDeclaration();
		if (hasFootnoteRules && !footnoteDeclaration.isEmpty()) {
			buf.append(createDeclaration("hr.footnoterule", footnoteDeclaration));
		}

		return buf.toString();
	}

	private String composePageDeclaration() {
		MasterPage masterPage = ofr.getFirstMasterPage();
		PageLayout pageLayout = ofr.getPageLayout(masterPage.getPageLayoutName());
		StyleInfo pageInfo = new StyleInfo();
		getFrameSc().cssBackground(pageLayout, pageInfo.props, true);
		String pageDeclaration = pageInfo.props.toString();
		return pageDeclaration;
	}

	private String composeFootnoteDeclaration() {
		MasterPage masterPage = ofr.getFirstMasterPage();
		PageLayout pageLayout = ofr.getPageLayout(masterPage.getPageLayoutName());
		StyleInfo ruleInfo = new StyleInfo();
		cssFootnoteRule(pageLayout, ruleInfo.props);
		String footnoteRulesDeclaration = ruleInfo.props.toString();
		return footnoteRulesDeclaration;
	}

		private String composeDeclaration(String displayName) {
			String declaration = "";
			String selector = composeSelector(displayName);
			String declarationBlock = composeDeclarationBlock(displayName);
			// Then export the results
			if (!selector.isEmpty() && !declarationBlock.isEmpty()) {
				declaration = createDeclaration(selector, declarationBlock);
			}
			return declaration;
		}

		private String composeDeclarationBlock( String displayName) {
			MasterPage style = (MasterPage) getStyles().getStyleByDisplayName(displayName);
			StyleInfo info = new StyleInfo();
			// First apply page layout (size)
			PageLayout pageLayout = ofr.getPageLayout(style.getPageLayoutName());
			if (pageLayout!=null) {
			    applyDirection(pageLayout,info);
			    cssPageSize(pageLayout,info.props);
			    getFrameSc().cssBackground(pageLayout,info.props,true);
			}
			// Next apply drawing-page style (draw background)
			StyleWithProperties drawingPage = ofr.getDrawingPageStyle(style.getProperty(XMLString.DRAW_STYLE_NAME));
			if (drawingPage!=null) {
			    cssDrawBackground(drawingPage,info.props,true);
			}
			return info.props.toString();
		}

		private String composeSelector( String displayName) {
			styleNames.addName(displayName);
			return ".masterpage" + styleNames.getName(displayName);
		}
	
    /** Get the family of page styles (master pages)
     *  @return the style family
     */
    public OfficeStyleFamily getStyles() {
        return ofr.getMasterPages();
    }

    // Background properties in draw: Color, gradient, hatching or bitmap
    private void cssDrawBackground(StyleWithProperties style, Properties props, boolean bInherit){
        // Fill color: Same as in css
        String s = style.getProperty(XMLString.DRAW_FILL_COLOR,bInherit);
        if (s!=null) { props.addProperty("background-color",s); }
    }

	
    private void cssPageSize(PageLayout style, Properties props) {
        String sWidth = style.getProperty(XMLString.FO_PAGE_WIDTH);
        if (sWidth!=null) { props.addProperty("width",scale(sWidth)); }
        String sHeight = style.getProperty(XMLString.FO_PAGE_HEIGHT);
        if (sHeight!=null) { props.addProperty("height",scale(sHeight)); }
    }
    
	// Footnote rule
    private void cssFootnoteRule(PageLayout style, Properties props) {
    	String sBefore = style.getFootnoteProperty(XMLString.STYLE_DISTANCE_BEFORE_SEP);
    	if (sBefore!=null) { props.addProperty("margin-top",scale(sBefore)); }
    	String sAfter = style.getFootnoteProperty(XMLString.STYLE_DISTANCE_AFTER_SEP);
    	if (sAfter!=null) { props.addProperty("margin-bottom", scale(sAfter)); }
    	String sHeight = style.getFootnoteProperty(XMLString.STYLE_WIDTH);
    	if (sHeight!=null) { props.addProperty("height", scale(sHeight)); }
    	String sWidth = style.getFootnoteProperty(XMLString.STYLE_REL_WIDTH);
    	if (sWidth!=null) { props.addProperty("width", sWidth); }
    	
    	String sColor = style.getFootnoteProperty(XMLString.STYLE_COLOR);
    	if (sColor!=null) { // To get the expected result in all browsers we must set both
    		props.addProperty("color", sColor);
    		props.addProperty("background-color", sColor);
    	}

    	String sAdjustment = style.getFootnoteProperty(XMLString.STYLE_ADJUSTMENT);
    	if ("right".equals(sAdjustment)) {
    		props.addProperty("margin-left", "auto");
    		props.addProperty("margin-right", "0");
    	}
    	else if ("center".equals(sAdjustment)) {
    		props.addProperty("margin-left", "auto");
    		props.addProperty("margin-right", "auto");
    	}
    	else { // default left
    		props.addProperty("margin-left", "0");
    		props.addProperty("margin-right", "auto");
    	}
    }


	
	
}
